@@ -20,21 +20,22 @@
 #    along with ReSiStance.  If not, see <http://www.gnu.org/licenses/>.
 #########################################################################
 
-import feedparser
-import gobject
+import base64
 import constants
 import cPickle
+import feedparser
+import gobject
+import gtk
+import os
 import urllib
 import urlparse
-import os
-import base64
-import gtk
+import xmlrpclib
+
 from settings import Settings
 from sgmllib import SGMLParser
 from threading import Thread
-from xml.dom.minidom import Document
 from xml.dom import minidom
-import xmlrpclib
+from xml.dom.minidom import Document
 
 # http://diveintomark.org/archives/2002/05/31/rss_autodiscovery_in_python
 def getRSSLink(url):
@@ -91,11 +92,16 @@ class ReSiStanceFeedDict(feedparser.FeedParserDict):
 
 class FeedManager(gobject.GObject):
 
-    def __init__(self, settings):
+    def __init__(self, settings, conn_manager):
         super(FeedManager, self).__init__()
 
         self.feed_data_list = []
         self.settings = settings
+        self._conn_manager = conn_manager
+        self._conn_manager.connect('connection-changed', self._on_connection_changed)
+
+    def _on_connection_changed(self, conn_manager):
+        pass
 
     def get_feed_list(self):
         return self.feed_data_list
@@ -152,7 +158,8 @@ class FeedManager(gobject.GObject):
         for feed_data in self.feed_data_list:
             outline = opml_doc.createElement('outline')
             outline.setAttribute('title',feed_data.feed.title)
-            outline.setAttribute('text',feed_data.feed.subtitle)
+            if 'subtitle' in feed_data.feed:
+                 outline.setAttribute('text',feed_data.feed.subtitle)
             outline.setAttribute('xmlUrl',feed_data['href'])
             body.appendChild(outline)
 
@@ -182,30 +189,39 @@ class FeedManager(gobject.GObject):
             callback(None, None)
 
     def _import_opml_in_thread(self, file_path, callback, user_data):
+
+        feed_url_list = [];
+
         if (not (os.path.exists(file_path) and os.path.isfile(file_path)) or
             (os.path.splitext(file_path)[1] != '.opml')):
-            gtk.gdk.threads_enter()
-            callback(None,None)
-            gtk.gdk.threads_leave()
+            if callback:
+                gtk.gdk.threads_enter()
+                callback(feed_url_list, user_data)
+                gtk.gdk.threads_leave()
             return
 
         try:
             doc=open(file_path,'r')
             opml_doc = minidom.parse(doc)
             opml_doc.getElementsByTagName('outline')
-            for node in opml_doc.getElementsByTagName('outline'):
-                if node.getAttribute('xmlUrl') != '':
-                    url = node.attributes['xmlUrl'].value
-                    # Add feed to manager
-                    self._add_feed_in_thread(url, callback, user_data)
+
+            feed_url_list = [node.attributes['xmlUrl'].value \
+                             for node in opml_doc.getElementsByTagName('outline') \
+                             if node.getAttribute('xmlUrl') != '']
         except IOError :
-            gtk.gdk.threads_enter()
-            callback(None,None)
-            gtk.gdk.threads_leave()
+            if callback:
+                gtk.gdk.threads_enter()
+                callback(feed_url_list, user_data)
+                gtk.gdk.threads_leave()
             return
 
         doc.close()
 
+        if callback:
+            gtk.gdk.threads_enter()
+            callback(feed_url_list, user_data)
+            gtk.gdk.threads_leave()
+
     def find_feed(self, key_words, dialog, callback):
         # Create a new thread to search
         find_thread = Thread(target=self._find_feed_in_thread,
@@ -311,7 +327,20 @@ class FeedManager(gobject.GObject):
             callback(data)
             gtk.gdk.threads_leave()
 
-    def get_favicon(self, url):
+    def get_favicon(self, url, callback, data=None):
+        get_favicon_thread = Thread(target=self._get_favicon_in_thread,
+                                    args=(url, callback, data))
+        get_favicon_thread.start()
+
+    def _get_favicon_in_thread(self, url, callback, data):
+        favicon = self._get_favicon_sync(url)
+
+        if callback:
+            gtk.gdk.threads_enter()
+            callback(favicon, data)
+            gtk.gdk.threads_leave()
+
+    def _get_favicon_sync(self, url):
         # Check that user dir exists
         user_path = os.path.join (constants.RSS_CONF_FOLDER, 'icons')
 
@@ -321,9 +350,12 @@ class FeedManager(gobject.GObject):
         file_name = os.path.join (user_path, base64.b64encode(url) + '.favicon.ico')
         if os.path.exists(file_name) == False:
             parsed_url = urlparse.urlsplit(url)
-            localfile, headers = urllib.urlretrieve(parsed_url.scheme + '://' +
-                                                    parsed_url.netloc + '/favicon.ico',
-                                                    file_name)
+            try:
+                localfile, headers = urllib.urlretrieve(parsed_url.scheme + '://' +
+                                                        parsed_url.netloc + '/favicon.ico',
+                                                        file_name)
+            except:
+                return None
 
             # Try with a more general address. If we got a text/html then we most
             # likely requested an invalid address. It's better this than to check
@@ -334,9 +366,12 @@ class FeedManager(gobject.GObject):
                 # Do not retry if domains == 2 because it will be
                 # the same address we tried before
                 if len(domains) > 2:
-                    localfile, headers = urllib.urlretrieve(parsed_url.scheme + '://' +
-                                                            domains[-2] + '.' + domains[-1] +
-                                                            '/favicon.ico', file_name)
+                    try:
+                        localfile, headers = urllib.urlretrieve(parsed_url.scheme + '://' +
+                                                                domains[-2] + '.' + domains[-1] +
+                                                                '/favicon.ico', file_name)
+                    except:
+                        return None
 
                 if headers['Content-type'].startswith('image/') == False:
                     os.remove(localfile)
@@ -372,9 +407,9 @@ class FeedManager(gobject.GObject):
         self.feed_data_list.append(new_feed_data)
 
         if 'link' in new_feed_data.feed:
-            pixbuf = self.get_favicon(new_feed_data.feed.link)
+            pixbuf = self._get_favicon_sync(new_feed_data.feed.link)
         else:
-            pixbuf = self.get_favicon(new_feed_data.href)
+            pixbuf = self._get_favicon_sync(new_feed_data.href)
 
         # Call user callback
         gtk.gdk.threads_enter()
