@@ -20,10 +20,8 @@
 #    along with ReSiStance.  If not, see <http://www.gnu.org/licenses/>.
 #########################################################################
 
-from feedmanager import FeedManager
 from pango import ELLIPSIZE_END
 from portrait import FremantleRotation
-from settings import Settings
 from threading import Thread
 from webkit import WebView
 import constants
@@ -51,9 +49,6 @@ default_color = color_style.lookup_color(constants.DEFAULT_TEXT_COLOR)
 secondary_color = color_style.lookup_color(constants.SECONDARY_TEXT_COLOR)
 del color_style
 
-settings = Settings()
-manager = FeedManager(settings)
-
 # From http://effbot.org/zone/re-sub.htm#unescape-html
 def unescape(text):
     def fixup(m):
@@ -134,11 +129,13 @@ class FeedsView(hildon.GtkTreeView):
 
 class SettingsDialog(gtk.Dialog):
 
-    def __init__(self, parent):
+    def __init__(self, parent, settings):
         super(SettingsDialog, self).__init__(_('Settings'), parent,
                                                 gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                                                 (gtk.STOCK_SAVE, gtk.RESPONSE_ACCEPT))
 
+        self.settings = settings
+
         # Orientation picker
         self.selector_orientation = hildon.TouchSelector(text=True)
         for caption in FremantleRotation.MODE_CAPTIONS:
@@ -179,20 +176,20 @@ class SettingsDialog(gtk.Dialog):
         self.picker_download_folder = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                       hildon.BUTTON_ARRANGEMENT_VERTICAL)
         self.picker_download_folder.set_title(_('Auto Download Folder'))
-        self.picker_download_folder.set_value(settings.auto_download_folder)
+        self.picker_download_folder.set_value(self.settings.auto_download_folder)
         self.picker_download_folder.set_alignment(0, 0.5, 0, 0)
         self.picker_download_folder.connect('clicked', self._download_folder_cb)
         self.picker_download_folder.set_style(hildon.BUTTON_STYLE_PICKER)
-        self.picker_download_folder.set_sensitive(settings.auto_download)
+        self.picker_download_folder.set_sensitive(self.settings.auto_download)
         self.vbox.add(self.picker_download_folder)
         self.picker_download_folder.show()
 
         # Load settings
-        self.auto_load_images.set_active(settings.auto_load_images)
-        self.automatic_download.set_active(settings.auto_download)
-        self.selector_orientation.set_active(0, settings.rotation_mode)
+        self.auto_load_images.set_active(self.settings.auto_load_images)
+        self.automatic_download.set_active(self.settings.auto_download)
+        self.selector_orientation.set_active(0, self.settings.rotation_mode)
         try:
-            font_size_index = constants.font_size_range.index(settings.default_font_size)
+            font_size_index = constants.font_size_range.index(self.settings.default_font_size)
         except ValueError:
             # defaults to 16pt
             font_size_index = constants.font_size_range.index(16)
@@ -208,7 +205,7 @@ class SettingsDialog(gtk.Dialog):
         chooser = hildon.FileChooserDialog(window, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
         chooser.set_transient_for(window)
         chooser.set_title(_('Select a folder for automatic downloads'))
-        chooser.set_current_folder('/home/user/MyDocs')
+        chooser.set_current_folder(constants.MYDOCS_DIR)
 
         chooser.set_default_response(gtk.RESPONSE_OK)
 
@@ -216,7 +213,7 @@ class SettingsDialog(gtk.Dialog):
         folder = chooser.get_filename() + '/'
         chooser.destroy()
 
-        settings.auto_download_folder = folder
+        self.settings.auto_download_folder = folder
         button.set_value(folder)
 
 class NewFeedDialog(gtk.Dialog):
@@ -278,9 +275,14 @@ class FeedsWindow(hildon.StackableWindow):
     FEEDS_MODEL_READ_COLUMN = 3
     FEEDS_MODEL_VISITS_COLUMN = 4
 
-    def __init__(self):
+    def __init__(self, manager, settings, conn_manager):
         super(FeedsWindow, self).__init__()
 
+        self.manager = manager
+        self.settings = settings
+        self._conn_manager = conn_manager
+        self._conn_manager.connect('connection-changed', self._on_connection_changed)
+
         # Feeds
         self.view = FeedsView(gtk.HILDON_UI_MODE_NORMAL)
         self.view.set_model (gtk.ListStore(gtk.gdk.Pixbuf, str, gobject.TYPE_PYOBJECT, str, int))
@@ -319,18 +321,33 @@ class FeedsWindow(hildon.StackableWindow):
         hildon.Program.get_instance().add_window(self)
 
         # Apply settings
-        settings.load()
-        self._sort(settings.feeds_order)
-        if settings.feeds_order == constants.DESCENDING_ORDER:
+        self.settings.load()
+        self._sort(self.settings.feeds_order)
+        if self.settings.feeds_order == constants.DESCENDING_ORDER:
             self.descending_filter_button.set_active(True)
-        elif settings.feeds_order == constants.VISITS_ORDER:
+        elif self.settings.feeds_order == constants.VISITS_ORDER:
             self.visits_filter_button.set_active(True)
 
+        # Force online/offline status evaluation
+        self._on_connection_changed(self._conn_manager)
+
         # Load Feeds (could be none)
         try:
-            manager.load(self._feed_data_loaded)
+            self.manager.load(self._feed_data_loaded)
         except IOError:
-            pass
+            self._show_initial_dialog()
+
+    def _on_connection_changed(self, conn_manager):
+        if self._conn_manager.is_online():
+            self._new_feed_button.show()
+            self._update_all_button.show()
+            self._find_feeds_button.show()
+            self._import_feeds_button.show()
+        else:
+            self._new_feed_button.hide()
+            self._update_all_button.hide()
+            self._find_feeds_button.hide()
+            self._import_feeds_button.hide()
 
     def _sort(self, order):
         if (order == constants.ASCENDING_ORDER):
@@ -370,7 +387,7 @@ class FeedsWindow(hildon.StackableWindow):
             hildon.hildon_banner_show_information(self, '', _('Wait until feed is refreshed'))
             return
 
-        news_window = EntriesWindow(feed_data)
+        news_window = EntriesWindow(feed_data, self.manager, self.settings, self._conn_manager)
         hildon.Program.get_instance().add_window(news_window)
         news_window.show()
 
@@ -401,8 +418,10 @@ class FeedsWindow(hildon.StackableWindow):
 
     def _on_exit(self, window, event):
         # Save the feeds status
-        manager.save(self._on_feed_data_saved)
-        settings.save()
+        self.hide_on_delete()
+        self.manager.save(self._on_feed_data_saved)
+        self.settings.save()
+        return True
 
     def _on_feed_data_saved(self, user_data):
         # Close application
@@ -432,26 +451,25 @@ class FeedsWindow(hildon.StackableWindow):
         self.visits_filter_button.connect('clicked', self._sort_visits_cb)
         menu.add_filter(self.visits_filter_button)
 
-
-        button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
-        button.set_label(_('New Feed'))
-        button.connect('clicked', self._new_feed_cb)
-        menu.append(button)
+        self._new_feed_button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self._new_feed_button.set_label(_('New Feed'))
+        self._new_feed_button.connect('clicked', self._new_feed_cb)
+        menu.append(self._new_feed_button)
 
         button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
         button.set_label(_('Remove Feed'))
         button.connect('clicked', self._remove_feed_cb)
         menu.append(button)
 
-        button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
-        button.set_label(_('Update all'))
-        button.connect('clicked', self._update_all_cb)
-        menu.append(button)
+        self._update_all_button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self._update_all_button.set_label(_('Update all'))
+        self._update_all_button.connect('clicked', self._update_all_cb)
+        menu.append(self._update_all_button)
 
-        button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
-        button.set_label(_('Settings'))
-        button.connect('clicked', self._preferences_cb)
-        menu.append(button)
+        self._import_feeds_button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self._import_feeds_button.set_label(_('Import Feeds'))
+        self._import_feeds_button.connect('clicked', self._import_feed_cb)
+        menu.append(self._import_feeds_button)
 
         button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
         button.set_label(_('Export Feeds'))
@@ -459,14 +477,14 @@ class FeedsWindow(hildon.StackableWindow):
         menu.append(button)
         self.export_opml_button = button
 
-        button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
-        button.set_label(_('Import Feeds'))
-        button.connect('clicked', self._import_feed_cb)
-        menu.append(button)
+        self._find_feeds_button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self._find_feeds_button.set_label(_('Find Feeds'))
+        self._find_feeds_button.connect('clicked', self._find_feed_cb)
+        menu.append(self._find_feeds_button)
 
         button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
-        button.set_label(_('Find Feeds'))
-        button.connect('clicked', self._find_feed_cb)
+        button.set_label(_('Settings'))
+        button.connect('clicked', self._preferences_cb)
         menu.append(button)
 
         menu.show_all()
@@ -506,8 +524,8 @@ class FeedsWindow(hildon.StackableWindow):
         removed = []
         for path in paths:
             try:
-                manager.remove_feed(model[path][FeedsView.FEEDS_DATA_COLUMN])
-                feed_data_list = manager.get_feed_list()
+                self.manager.remove_feed(model[path][FeedsView.FEEDS_DATA_COLUMN])
+                feed_data_list = self.manager.get_feed_list()
                 if len(feed_data_list) == 0:
                     self.export_opml_button.hide()
             except IOError:
@@ -532,7 +550,7 @@ class FeedsWindow(hildon.StackableWindow):
         self.unfullscreen()
 
     def _preferences_cb(self, button):
-        dialog = SettingsDialog(self)
+        dialog = SettingsDialog(self, self.settings)
         dialog.connect('response', self._preferences_response_cb)
         dialog.show_all()
 
@@ -541,7 +559,7 @@ class FeedsWindow(hildon.StackableWindow):
         hildon.hildon_gtk_window_set_progress_indicator(self, True)
 
         # Ask manager to update feed
-        manager.update_all(self._all_feeds_updated_cb, None)
+        self.manager.update_all(self._all_feeds_updated_cb, None)
 
     def _get_visual_unread_text(self, unread_count):
         if unread_count:
@@ -574,13 +592,13 @@ class FeedsWindow(hildon.StackableWindow):
             row_reference = gtk.TreeRowReference(self.view.get_model(), path)
 
             # Add feed to manager
-            manager.add_feed(url, self._feed_added_cb, row_reference)
+            self.manager.add_feed(url, self._feed_added_cb, row_reference)
 
             dialog.destroy()
 
     def _feed_added_cb(self, pixbuf, new_feed_data, row_reference=None):
 
-        feed_data_list = manager.get_feed_list()
+        feed_data_list = self.manager.get_feed_list()
         if len(feed_data_list) > 0:
             self.export_opml_button.show()
 
@@ -618,15 +636,15 @@ class FeedsWindow(hildon.StackableWindow):
     def _preferences_response_cb(self, dialog, response):
 
         if response == gtk.RESPONSE_ACCEPT:
-            settings.auto_load_images = dialog.auto_load_images.get_active()
-            settings.rotation_mode = dialog.selector_orientation.get_active(0)
-            settings.default_font_size = int(dialog.selector_font_size.get_current_text())
-            settings.auto_download = dialog.automatic_download.get_active()
+            self.settings.auto_load_images = dialog.auto_load_images.get_active()
+            self.settings.rotation_mode = dialog.selector_orientation.get_active(0)
+            self.settings.default_font_size = int(dialog.selector_font_size.get_current_text())
+            self.settings.auto_download = dialog.automatic_download.get_active()
 
             dialog.destroy()
 
             # Update rotation
-            self.rotation_object.set_mode(settings.rotation_mode)
+            self.rotation_object.set_mode(self.settings.rotation_mode)
 
     def _feed_exported_cb(self, exported):
         if exported:
@@ -638,12 +656,19 @@ class FeedsWindow(hildon.StackableWindow):
         hildon.hildon_gtk_window_set_progress_indicator(self, False)
 
     def _initial_dialog_new_cb(self, button, dialog):
-        dialog.destroy()
+        dialog.hide()
         self._new_feed_cb(button)
+        dialog.destroy()
+
+    def _initial_dialog_import_cb(self, button, dialog):
+        dialog.hide()
+        self._import_feed_cb(button)
+        dialog.destroy()
 
     def _initial_dialog_find_cb(self, button, dialog):
-        dialog.destroy()
+        dialog.hide()
         self._find_feed_cb(button)
+        dialog.destroy()
 
     def _show_initial_dialog(self):
         dialog = gtk.Dialog()
@@ -654,21 +679,27 @@ class FeedsWindow(hildon.StackableWindow):
         add_feed_button.set_label(_('New feed'))
         add_feed_button.connect('clicked', self._initial_dialog_new_cb, dialog)
 
+        import_feed_button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        import_feed_button.set_label(_('Import Feeds'))
+        import_feed_button.connect('clicked', self._initial_dialog_import_cb, dialog)
+
         find_feed_button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
         find_feed_button.set_label(_('Find Feeds'))
         find_feed_button.connect('clicked', self._initial_dialog_find_cb, dialog)
 
         dialog.vbox.add(add_feed_button)
+        dialog.vbox.add(import_feed_button)
         dialog.vbox.add(find_feed_button)
 
         add_feed_button.show()
+        import_feed_button.show()
         find_feed_button.show()
         dialog.show()
 
     def _feed_data_loaded(self, user_data):
         # TODO: migrate to "with" statement when available
         try:
-            feed_data_list = manager.get_feed_list()
+            feed_data_list = self.manager.get_feed_list()
             if len(feed_data_list) == 0:
                 self.export_opml_button.hide()
         except IOError:
@@ -678,19 +709,30 @@ class FeedsWindow(hildon.StackableWindow):
 
             for feed_data in feed_data_list:
                 feed_iter = store.append()
-                pixbuf = manager.get_favicon(feed_data.feed.link or feed_data.href)
                 unread_count = len([entry for entry in feed_data.entries if entry.read == False])
                 store.set(feed_iter,
-                          self.FEEDS_MODEL_PIXBUF_COLUMN, pixbuf,
                           self.FEEDS_MODEL_TITLE_COLUMN, self.view.get_feed_title_markup(feed_data),
                           self.FEEDS_MODEL_DATA_COLUMN, feed_data,
                           self.FEEDS_MODEL_READ_COLUMN, self._get_visual_unread_text(unread_count),
                           self.FEEDS_MODEL_VISITS_COLUMN, feed_data['visits'])
+                # Favicon might require a network download
+                path = self.view.get_model().get_path(feed_iter)
+                row_reference = gtk.TreeRowReference(store, path)
+                pixbuf = self.manager.get_favicon(feed_data.feed.link or feed_data.href,
+                                             self._get_favicon_cb, row_reference)
 
         # If there is no data then show a dialog
         if len(feed_data_list) == 0:
             self._show_initial_dialog()
 
+    def _get_favicon_cb(self, pixbuf, row_reference):
+        if pixbuf == None:
+            return
+
+        store = self.view.get_model()
+        feed_iter = store.get_iter(row_reference.get_path())
+        store.set(feed_iter, self.FEEDS_MODEL_PIXBUF_COLUMN, pixbuf)
+
     def _find_feed_response_cb(self, dialog, response):
         if response == gtk.RESPONSE_ACCEPT:
             key_words = dialog.entry.get_text()
@@ -698,7 +740,7 @@ class FeedsWindow(hildon.StackableWindow):
             # Quickly show feedback to user
             hildon.hildon_gtk_window_set_progress_indicator(dialog, True)
 
-            manager.find_feed(key_words, dialog, self._found_cb)
+            self.manager.find_feed(key_words, dialog, self._found_cb)
             dialog.set_response_sensitive(gtk.RESPONSE_ACCEPT, False)
 
             # Disable canceling
@@ -716,7 +758,7 @@ class FeedsWindow(hildon.StackableWindow):
             row_reference = gtk.TreeRowReference(self.view.get_model(), path)
 
             # Add feed to manager
-            manager.add_feed(url, self._feed_added_cb, row_reference)
+            self.manager.add_feed(url, self._feed_added_cb, row_reference)
 
     def _found_cb(self, key_words, dialog, feeds_info):
         if feeds_info == None:
@@ -737,15 +779,15 @@ class FeedsWindow(hildon.StackableWindow):
         dialog.destroy()
 
     def _sort_ascending_cb(self, button):
-        settings.feeds_order = constants.ASCENDING_ORDER
+        self.settings.feeds_order = constants.ASCENDING_ORDER
         self._sort(constants.ASCENDING_ORDER)
 
     def _sort_descending_cb(self, button):
-        settings.feeds_order = constants.DESCENDING_ORDER
+        self.settings.feeds_order = constants.DESCENDING_ORDER
         self._sort(constants.DESCENDING_ORDER)
 
     def _sort_visits_cb(self, button):
-        settings.feeds_order = constants.VISITS_ORDER
+        self.settings.feeds_order = constants.VISITS_ORDER
         self._sort(constants.VISITS_ORDER)
 
     def _export_feed_cb(self, button):
@@ -755,7 +797,7 @@ class FeedsWindow(hildon.StackableWindow):
         chooser = hildon.FileChooserDialog(window, gtk.FILE_CHOOSER_ACTION_SAVE)
         chooser.set_transient_for(window)
         # Don't like this at all. Isn't there any define in the platform ?
-        chooser.set_current_folder('/home/user/MyDocs/.documents')
+        chooser.set_current_folder(constants.MYDOCS_DIR)
         chooser.set_current_name('resistance-feeds')
 
         #Overwrite the file if already exists
@@ -765,7 +807,7 @@ class FeedsWindow(hildon.StackableWindow):
         response = chooser.run()
         if response == gtk.RESPONSE_OK:
             hildon.hildon_gtk_window_set_progress_indicator(self, True)
-            manager.export_opml(chooser.get_filename() + '.opml', self._feed_exported_cb)
+            self.manager.export_opml(chooser.get_filename() + '.opml', self._feed_exported_cb)
         elif response == gtk.RESPONSE_CANCEL:
             print 'Closed, no files selected'
         chooser.destroy()
@@ -778,7 +820,7 @@ class FeedsWindow(hildon.StackableWindow):
         chooser = hildon.FileChooserDialog(window ,gtk.FILE_CHOOSER_ACTION_OPEN)
         chooser.set_transient_for(window)
         # Don't like this at all. Isn't there any define in the platform ?
-        chooser.set_current_folder('/home/user/MyDocs/.documents')
+        chooser.set_current_folder(constants.MYDOCS_DIR)
         chooser.set_default_response(gtk.RESPONSE_OK)
 
         #Filter for opml files
@@ -790,12 +832,27 @@ class FeedsWindow(hildon.StackableWindow):
         response = chooser.run()
         if response == gtk.RESPONSE_OK:
             hildon.hildon_gtk_window_set_progress_indicator(self, True)
-            manager.import_opml(chooser.get_filename(), self._feed_added_cb)
+            self.manager.import_opml(chooser.get_filename(), self._import_opml_cb)
         elif response == gtk.RESPONSE_CANCEL:
             print 'Closed, no files selected'
 
         chooser.destroy()
 
+    def _import_opml_cb(self, feed_url_list, data):
+        if len(feed_url_list) == 0:
+            return
+
+        hildon.hildon_gtk_window_set_progress_indicator(self, True)
+
+        for feed_url in feed_url_list:
+            # Insert a dummy row while information is retrieved
+            dummy_feed_data, feed_iter = self._add_dummy_feed(feed_url)
+            path = self.view.get_model().get_path(feed_iter)
+            row_reference = gtk.TreeRowReference(self.view.get_model(), path)
+
+            # Add feed to manager
+            self.manager.add_feed(feed_url, self._feed_added_cb, row_reference)
+
     def _find_feed_cb(self, button):
         dialog = FindFeedsDialog(self)
         dialog.connect('response', self._find_feed_response_cb)
@@ -886,12 +943,16 @@ class EntriesWindow(hildon.StackableWindow):
     ENTRIES_MODEL_DATA_COLUMN = 2
     ENTRIES_MODEL_DATE_COLUMN = 3
 
-    def __init__(self, feed_data):
+    def __init__(self, feed_data, manager, settings, conn_manager):
         super(EntriesWindow, self).__init__()
 
+        self.manager = manager
+        self.settings = settings
         self.feed_data = feed_data
-        self.set_title(unescape(self.feed_data.feed.title))
+        self._conn_manager = conn_manager
+        self._conn_manager.connect('connection-changed', self._on_connection_changed)
 
+        self.set_title(unescape(self.feed_data.feed.title))
         self.toolbar = gtk.Toolbar()
 
         # Edit toolbar
@@ -928,22 +989,34 @@ class EntriesWindow(hildon.StackableWindow):
 
         # Update feed button
         action_box = hildon.GtkTreeView.get_action_area_box(self.view)
-        update_feed_button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
-        update_feed_button.set_label(_('Update feed'))
-        update_feed_button.connect('clicked', self._update_feed_cb)
+        self._update_feed_button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self._update_feed_button.set_label(_('Update feed'))
+        self._update_feed_button.connect('clicked', self._update_feed_cb)
 
-        action_box.pack_start(update_feed_button, True, True)
-        update_feed_button.show()
-        hildon.GtkTreeView.set_action_area_visible(self.view, True)
+        action_box.pack_start(self._update_feed_button, True, True)
 
         self._create_menu()
 
         if 'itunes' not in self.feed_data.namespaces:
-            self.download_all_button.hide()
-
+            self._download_all_button.hide()
 
         self.connect('configure-event', self._on_configuration_changed)
 
+        # Force online/offline status evaluation
+        self._on_connection_changed(self._conn_manager)
+
+    def _on_connection_changed(self, conn_manager):
+        if conn_manager.is_online():
+            self._update_feed_button.show()
+            # Check that we should not show it
+            if 'itunes' in self.feed_data.namespaces:
+                self._download_all_button.show()
+            hildon.GtkTreeView.set_action_area_visible(self.view, True)
+        else:
+            self._update_feed_button.hide()
+            self._download_all_button.hide()
+            hildon.GtkTreeView.set_action_area_visible(self.view, False)
+
     def _create_menu(self):
         menu = hildon.AppMenu()
 
@@ -957,11 +1030,10 @@ class EntriesWindow(hildon.StackableWindow):
         button.connect('clicked', self._mark_read_cb, False)
         menu.append(button)
 
-        button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
-        button.set_label(_('Download all'))
-        button.connect('clicked', self._download_all_items_cb)
-        menu.append(button)
-        self.download_all_button = button
+        self._download_all_button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self._download_all_button.set_label(_('Download all'))
+        self._download_all_button.connect('clicked', self._download_all_items_cb)
+        menu.append(self._download_all_button)
 
         menu.show_all()
         self.set_app_menu(menu)
@@ -1011,7 +1083,7 @@ class EntriesWindow(hildon.StackableWindow):
         self.unfullscreen()
 
     def _on_entry_activated(self, treeview, path, column):
-        item_window = ItemWindow(self.feed_data)
+        item_window = ItemWindow(self.feed_data, self.manager, self.settings, self._conn_manager)
         hildon.Program.get_instance().add_window(item_window)
 
         item_window.connect('item-read', self._item_read_cb)
@@ -1031,12 +1103,14 @@ class EntriesWindow(hildon.StackableWindow):
 
         # Quickly show feedback to user
         hildon.hildon_gtk_window_set_progress_indicator(self, True)
+        button.set_sensitive(False)
 
         # Ask manager to update feed
-        manager.update_feed(self.feed_data, self._feed_updated_cb, None)
+        self.manager.update_feed(self.feed_data, self._feed_updated_cb, None)
 
     def _feed_updated_cb(self, data):
         hildon.hildon_gtk_window_set_progress_indicator(self, False)
+        self._update_feed_button.set_sensitive(True)
 
         # No need to update self.feed_data as it is automatically updated.
         # The same happens with the reference stored in the FeedsWindow :-)
@@ -1053,7 +1127,7 @@ class EntriesWindow(hildon.StackableWindow):
         return
 
     def _download_all_items_cb(self, button):
-        folder = settings.auto_download_folder
+        folder = self.settings.auto_download_folder
 
         urls = []
         paths_files = []
@@ -1067,7 +1141,7 @@ class EntriesWindow(hildon.StackableWindow):
 
         hildon.hildon_gtk_window_set_progress_indicator(self, True)
 
-        manager.download_all_items(urls, paths_files, self._all_items_downloaded_cb)
+        self.manager.download_all_items(urls, paths_files, self._all_items_downloaded_cb)
 
     def _all_items_downloaded_cb(self, downloaded):
         if downloaded:
@@ -1141,10 +1215,14 @@ class ItemWindow(hildon.StackableWindow):
         "item-read": (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT, )),
         }
 
-    def __init__(self, feed_data):
+    def __init__(self, feed_data, manager, settings, conn_manager):
         super(ItemWindow, self).__init__()
 
+        self.manager = manager
+        self.settings = settings
         self.feed_data = feed_data
+        self._conn_manager = conn_manager
+        self._conn_manager.connect('connection-changed', self._on_connection_changed)
 
         # Header
         self.item_window_header = ItemWindowHeader(False, 16)
@@ -1160,8 +1238,8 @@ class ItemWindow(hildon.StackableWindow):
 
         # Set some settings
         wbk_settings = self.view.get_settings()
-        wbk_settings.set_property('default_font_size', settings.default_font_size)
-        wbk_settings.set_property('auto-load-images', settings.auto_load_images)
+        wbk_settings.set_property('default_font_size', self.settings.default_font_size)
+        wbk_settings.set_property('auto-load-images', self.settings.auto_load_images)
         wbk_settings.set_property('auto-shrink-images', True)
 
         # Pannable for showing content
@@ -1189,8 +1267,16 @@ class ItemWindow(hildon.StackableWindow):
         self._create_menu()
         self._create_toolbar()
 
+        # For portrait/landscape modes
         self.connect('configure-event', self._on_configuration_changed)
 
+    def _on_connection_changed(self, conn_manager):
+        if conn_manager.is_online():
+            if 'enclosures' in self.item_window_header.item:
+                self._download_button.show()
+        else:
+            self._download_button.hide()
+
     def _on_configuration_changed(self, window, event):
 
         # Show toolbar in portrait mode. Show header in landscape mode
@@ -1250,18 +1336,18 @@ class ItemWindow(hildon.StackableWindow):
         button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
         button.set_label(_('Item Details'))
         button.connect('clicked', self._item_details_cb)
+        button.show()
         menu.append(button)
 
-        button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
-        button.set_label(_('Download'))
-        button.connect('clicked', self._download_item_cb)
-        menu.append(button)
-        self.download_button = button
+        self._download_button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self._download_button.set_label(_('Download'))
+        self._download_button.connect('clicked', self._download_item_cb)
+        self._download_button.show()
+        menu.append(self._download_button)
 
         menu.show_all()
         self.set_app_menu(menu)
 
-
     def _add_to_table (self, table, color, key, value, y_coord):
         label = gtk.Label('')
         label.set_markup('<span foreground="%s">%s:</span>' % (color, key))
@@ -1333,7 +1419,6 @@ class ItemWindow(hildon.StackableWindow):
         item_iter = model.get_iter(path)
         item = model.get_value(item_iter, EntriesWindow.ENTRIES_MODEL_DATA_COLUMN)
 
-        self.item_window_header.set_item(item)
         self.set_item(item)
 
     def set_item(self, item):
@@ -1343,10 +1428,6 @@ class ItemWindow(hildon.StackableWindow):
             author = self.feed_data.feed.title
         self.set_title(author)
 
-        # hide the button if is not a podcast
-        if 'enclosures' not in item:
-            self.download_button.hide()
-
         # Set item in header
         self.item_window_header.set_item(item)
 
@@ -1390,16 +1471,17 @@ class ItemWindow(hildon.StackableWindow):
         # Emit item read
         self.emit('item-read', self.model.get_path(self.model_iter))
 
+        # Force online/offline status evaluation
+        self._on_connection_changed(self._conn_manager)
+
     def _download_item_cb(self, button):
         item = self.item_window_header.item
         url_file = item.enclosures[0]['href']
         file = os.path.basename(urllib.url2pathname(url_file))
-        folder = settings.auto_download_folder
+        folder = self.settings.auto_download_folder
 
         hildon.hildon_gtk_window_set_progress_indicator(self, True)
-        manager.download_item(url_file, folder+file, self._item_downloaded_cb)
-
-
+        self.manager.download_item(url_file, folder+file, self._item_downloaded_cb)
 
     def _item_downloaded_cb(self, downloaded):
         if downloaded:
